; disp2.asm - Example program using custom video driver. Paul Daniels

; 21/04/2010 - PD Initial version

; Character set

;  ____0___1___2___3___4___5___6___7___8___9___A___B___C___D___E___F____
;  00 SPC #_  _#  ##  __  #_  _#  ##  HH  __  HH   "  GBP  $   :   ?  0F
;         __  __  __  #_  #_  #_  #_  HH  HH  __

;  10  (   )   >   <   =   +   -   *   /   ;   ,   .   0   1   2   3  1F
;  20  4   5   6   7   8   9   A   B   C   D   E   F   G   H   I   J  2F
;  30  K   L   M   N   O   P   Q   R   S   T   U   V   W   X   Y   Z  3F

; Keyboard matrix when reading LAST_K

; LAST_K+1     FD  FB  F7  EF  DF | DF  EF  F7  FB  FD
;                                 |
; LAST_K  BIT                     |                    BIT
; F7       3   1   2   3   4   5  | 6   7   8   9   0   4  EF
;                                 |
; FB       2   Q   W   E   R   T  | Y   U   I   O   P   5  DF
;                                 
; FD       1   A   S   D   F   G  | H   J   K   L  ret  6  BF
;                                 |
; FE       0       Z   X   C   V  | B   N   M   .  spc  7  7F

; Shift is bit 0 on LAST_K + 1.

	.INCLUDE "zx81_16k.inc"

.SECTION "PFILE"

; Format of P file with no variables

; Once new driver is running, memory from $4000 to $404a, except D_FILE
; will be available.

	.EQU	ERR_NR	$4000
	.EQU	FLAGS	$4001
	.EQU	ERR_SP	$4002
	.EQU	RAMTOP	$4004
	.EQU	MODE	$4006
	.EQU	PPC	$4007
        .EXPORT ERR_NR, FLAGS, ERR_SP, RAMTOP, MODE, PPC

VERSN	.DB $00
E_PPC	.DW $0000
D_FILE	.DW FRAME0
DF_CC	.DW FRAME0 + 1
VARS	.DW VARAREA
DEST	.DW $0000
E_LINE	.DW VARAREA + 1
CH_ADD	.DW AUTO - 1
X_PTR	.DW $0000
STKBOT	.DW VARAREA + 2
STKEND	.DW VARAREA + 2
BERG	.DB $00
MEM	.DW $405D
QF_FLG	.DB $00
DF_SZ	.DB $02
S_TOP	.DW $0000
LAST_K	.DW $FFFF
DBOUNCE	.DB $00
MARGIN	.DB $37
NXTLIN	.DW AUTO
OLDPPC	.DW $0000
FLAGX	.DB $00
STRLEN	.DW $0000
T_ADDR	.DW $0C8D
SEED	.DW $0000
FRAMES	.DW $0000
COORDS	.DW $0000
PR_CC	.DB $BC
S_POSN	.DB $21
LEVEL	.DB $18
CDFLAG	.DB $40
;PRBUFF	.DS 32, 0
;	.DB $76
;MEMBOT	.DS 30, 0
;	.DW $0000

;*******************************************************************************
; Defines
;*******************************************************************************

;*******************************************************************************
; Basic program
;*******************************************************************************

	.DB $76			; Trick Basic to think this is the previous line

; 0 PRINT USR 0

AUTO	.DW 0				; Line number (MSB)
	.DW USREND - USRBEG		; Line length
USRBEG	.DB $f5, $d4,			; PRINT USR
	.ASC "0"                        ; Needs at least one character
	.DB 126,143,0,150,0,0           ; 16459
	.DB $76				; EOL
USREND

;*******************************************************************************
; Initialise Machine
;*******************************************************************************

	ld hl, $8000			; Init MC stack	to top of memory
	ld sp, hl

        ld hl, 31                       ; Init for ISR
        ld (BARCNT), hl
        xor a
        ld (BARINT), a
        ld hl, ISR
        call D_TISR                     ; Set ISR in Top margin.

        call D_START                    ; Start the ball rolling

        ld hl, FRAME1                   ; Use a second display file.
        call D_F1ADDR

	ld bc, 1024
-	call D_TWAIT
	dec bc
	ld a, c
	or b
	jr nz, -

	ld hl,0				; Turn off ISR
	call D_TISR
        ld hl, FRAME0			; Set both frames the same
        call D_F1ADDR

	; Clear frame 1

	ld hl, FRAME0
	ld bc, 24 * 33
-	ld a, (hl)
	cp 118
	jr z, +
	ld (hl), 0
+
	inc hl
	dec bc
	ld a, c
	or b
	jr nz, -

	; Show speed increase with smaller frames

	ld de, FRAME0 + 1 + (12 * 33) + 2
	ld hl, TXT1
	ld bc, 28
	ldir

	ld hl, 20
-       ld de, FRAME0 + 1 + (12 * 33) + 29
	call C_DOWN
	dec hl
	ld a, l
	or h
	jr nz, -

	; Reduce frame size

	ld hl, $0c02			; offset 12, length 2
	call D_F0SET
	call D_F1SET

	call D_BWAIT
	ld de, FRAME0 + 1 + (0 * 33) + 2
	ld hl, FRAME0 + 1 + (12 * 33) + 2
	ld bc, 28
	ldir

	ld de, FRAME0 + 1 + (1 * 33) + 2
	ld hl, TXT2
	ld bc, 27
	ldir

	ld hl, 20
-       ld de, FRAME0 + 1 + (0 * 33) + 29
	call C_DOWN
	dec hl
	ld a, l
	or h
	jr nz, -

	ld a, 2
	call D_SETFR

	ld de, FRAME0 + 1 + (1 * 33) + 2
	ld hl, TXT3
	ld bc, 28
	ldir

	ld hl, 20
-       ld de, FRAME0 + 1 + (0 * 33) + 29
	call C_DOWN
	dec hl
	ld a, l
	or h
	jr nz, -

	; Show game mockup screen

	ld hl, FGAME
	call D_F0ADDR
	call D_F1ADDR
G3	ld hl, $0000
	call D_F0SET
	inc l
	call D_F1SET
	ld a, 3
	call D_SETFR

	ld l, 1
G1	call D_F0SET
	inc l
	call D_F1SET
	ld b,5
G2	call D_TWAIT
	djnz G2
	ld a,l
	cp 21
	jr nz, G1
	ld l, $18
	call D_F1SET

-	jr -

C_DOWN
	ld bc, $801
SC0     ld a, (de)
        add a,c
        ld c,1
        cp 38                           ; 'A'
        jr nz, SC1
        ld a, 28                        ; '0'
SC1     ld (de),a
        jr nz, SC2
        dec de
        djnz SC0

SC2     ld b,0                          ; Delay
SC4     ld c,64
SC3     dec c
        jr nz,SC3
        djnz SC4

        ret

        ; This section flashes the whole screen grey and is a bit
        ; annoying.

;        call D_EWAIT                    ; Wait for an even frame
;        call D_TWAIT                    ; Wait for top of next odd frame
;LP0     ld a, 1
;        call D_SETFR
;        ld b, 16
;-       call D_TWAIT
;        djnz -
;        ld a, 3
;        call D_SETFR
;        ld b, 16
;-       call D_TWAIT
;        djnz -
;        jr LP0

TXT1	.ASC "DECOMPRESSING LEVEL 00000000"
TXT2	.ASC "LETS REDUCE THE FRAME SIZE."
TXT3	.ASC "LETS BLANK A COMPLETE FRAME."

;*******************************************************************************
; ISR
;*******************************************************************************
ISR
        ld a, (BARINT)                  ; Delay time bar
        inc a
        ld (BARINT), a
        cp 5
        ret nz

        xor a                           ; Reset time interval
        ld (BARINT), a

        ld de, (BARCNT)                 ; Get current bar char
        ld hl, BAR
        add hl,de
        ld a, (hl)

        cp $80
        jr z, FULLBLK

        cp $05
        jr z, LEFTBLK

        cp $00
        jr z, BLNKBLK

        ; Default is $85

RIGHBLK
        ld a, $80
        jr +

LEFTBLK
        ld a, $00
+       ld (hl), a

        ld a, (BARCNT)
        dec a
        cp $ff
        jr nz, +
        ld a, 31
        
+       ld (BARCNT), a
        ret

FULLBLK
        ld a, $05
        ld (hl), a
        ret

BLNKBLK
        ld a, $85
        ld (hl), a
        ret

BARINT  .DB 0
BARCNT  .DW 0

;*******************************************************************************
; Includes
;*******************************************************************************

	.INCLUDE "video.inc"

;*******************************************************************************
; Pre defined display file.                                                    *
;*******************************************************************************

FRAME0	.DB $76
        .ASC "@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@"
        .DB $76
        .ASC "@@@@@@@@@@blast@off@and@strike@@"
        .DB $76
        .ASC "@@@@@@@@@@the@evil@bydo@empire@@"
        .DB $76
        .ASC "@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@"
        .DB $76
        .ASC "@@@@@@@@@@@@@push@space@button@@"
        .DB $76
        .ASC "@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@"
        .DB $76
        .ASC "@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@"
        .DB $76
        .ASC "@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@"
        .DB $76
        .ASC "@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@"
        .DB $76
        .ASC "@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@"
        .DB $76
	.DB $80, $80, $80, $80, $80, $80, $80, $80, $80, $80, $80, $80, $80, $80, $80, $80, $80, $80, $80, $80, $80, $80, $80, $80, $80, $80, $80, $80, $80, $80, $80, $80
        .DB $76
	.DB $80, $07, $03, $03, $03, $80, $80, $80, $80, $80, $03, $03, $03, $03, $80, $03, $80, $80, $80, $03, $80, $03, $03, $03, $84, $80, $07, $03, $03, $03, $84, $80
        .DB $76
	.DB $80, $00, $87, $83, $04, $02, $80, $80, $80, $82, $83, $04, $87, $83, $81, $04, $02, $80, $01, $87, $05, $00, $83, $83, $00, $84, $00, $87, $83, $83, $83, $80
        .DB $76
	.DB $80, $00, $85, $80, $07, $00, $80, $07, $03, $80, $80, $05, $85, $80, $80, $80, $04, $00, $87, $80, $05, $00, $80, $80, $01, $85, $00, $85, $80, $80, $80, $80
        .DB $76
	.DB $80, $00, $85, $01, $00, $81, $80, $00, $06, $85, $80, $05, $85, $80, $80, $80, $80, $00, $80, $80, $05, $00, $07, $00, $87, $80, $00, $00, $00, $84, $80, $80
        .DB $76
	.DB $80, $00, $85, $00, $84, $80, $80, $06, $00, $81, $80, $05, $85, $80, $80, $80, $80, $00, $80, $80, $05, $00, $80, $80, $80, $80, $00, $85, $80, $80, $80, $80
        .DB $76
	.DB $80, $00, $85, $04, $00, $84, $80, $80, $80, $80, $80, $05, $85, $80, $80, $80, $80, $00, $80, $80, $05, $00, $80, $80, $80, $80, $00, $85, $80, $80, $80, $80
        .DB $76
	.DB $80, $00, $85, $80, $04, $00, $80, $80, $80, $80, $80, $05, $85, $80, $80, $80, $80, $00, $80, $80, $05, $00, $80, $80, $80, $80, $04, $00, $00, $00, $02, $80
        .DB $76
	.DB $80, $82, $80, $80, $80, $80, $80, $80, $80, $80, $80, $82, $80, $80, $80, $80, $80, $81, $80, $80, $80, $81, $80, $80, $80, $80, $80, $80, $80, $80, $80, $80
        .DB $76
        .ASC "@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@"
        .DB $76
        .ASC "@@@copyright@c@1987@irem@corp@@@"
        .DB $76
        .ASC "@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@"
        .DB $76
        .ASC "@@reprogrammed@game@c@???@2010@@"
        .DB $76
BAR
        .ASC "@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@"
        .DB $76

FRAME1	.DB $76
        .ASC "@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@"
        .DB $76
        .ASC "@@@@@@@@@@blast@off@and@strike@@"
        .DB $76
        .ASC "@@@@@@@@@@the@evil@bydo@empire@@"
        .DB $76
        .ASC "@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@"
        .DB $76
        .ASC "@@@@@@@@@@@@@push@space@button@@"
        .DB $76
        .ASC "@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@"
        .DB $76
        .ASC "@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@"
        .DB $76
        .ASC "@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@"
        .DB $76
        .ASC "@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@"
        .DB $76
        .ASC "@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@"
        .DB $76
        .ASC "@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@"
        .DB $76
        .ASC "@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@"
        .DB $76
        .ASC "@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@"
        .DB $76
        .ASC "@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@"
        .DB $76
        .ASC "@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@"
        .DB $76
        .ASC "@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@"
        .DB $76
        .ASC "@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@"
        .DB $76
        .ASC "@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@"
        .DB $76
        .ASC "@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@"
        .DB $76
        .ASC "@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@"
        .DB $76
        .ASC "@@@copyright@c@1987@irem@corp@@@"
        .DB $76
        .ASC "@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@"
        .DB $76
        .ASC "@@reprogrammed@game@c@???@2010@@"
        .DB $76
        .ASC "this@bar@handled@by@an@interrupt"
        .DB $76

	; Mock up game screen

	.DEFINE blank $80,$80,$80,$80
	.DEFINE splt1 $89,$89,$89,$89
	.DEFINE splt2 $8a,$8a,$8a,$8a

	.DEFINE flrA1 $0a,$0a,$0a,$89
	.DEFINE flrA2 $18,$10,$18,$11

	.DEFINE skyA1 $18,$10,$18,$11
	.DEFINE skyA2 $8a,$09,$09,$09

	.DEFINE flrB1 $89,$0a,$0a,$89
	.DEFINE flrB2 $05,$00,$12,$85
	.DEFINE flrB3 $05,$18,$0b,$85
	.DEFINE flrB4 $05,$0e,$0e,$85

	.DEFINE jetA1 $07,$80,$07,$03
	.DEFINE jetA2 $07,$81,$85,$89
	.DEFINE jetA3 $05,$85,$85,$88
	.DEFINE jetA4 $05,$81,$06,$03

	.DEFINE jetB1 $03,$03,$03,$84
	.DEFINE jetB2 $89,$07,$89,$80
	.DEFINE jetB3 $0a,$05,$88,$81
	.DEFINE jetB4 $03,$03,$03,$84

	.DEFINE ship1 $07,$03,$80,$80
	.DEFINE ship2 $92,$09,$02,$04
	.DEFINE ship3 $80,$82,$81,$80
	.DEFINE ship4 $84,$80,$80,$80

FGAME	.DB $76
	.DB blank, blank, blank, jetA1, jetB1, skyA1, skyA1, skyA1, $76
	.DB blank, blank, blank, jetA2, jetB2, skyA2, skyA2, skyA2, $76
	.DB blank, blank, blank, jetA3, jetB3, blank, blank, blank, $76
	.DB blank, blank, blank, jetA4, jetB4, blank, blank, blank, $76
	.DB blank, blank, blank, jetA1, jetB1, blank, blank, blank, $76
	.DB blank, blank, blank, jetA2, jetB2, blank, blank, blank, $76
	.DB blank, ship1, blank, jetA3, jetB3, blank, blank, blank, $76
	.DB blank, ship2, ship4, jetA4, jetB4, blank, blank, blank, $76
	.DB blank, ship3, blank, blank, blank, blank, blank, blank, $76
	.DB blank, blank, blank, blank, blank, blank, blank, blank, $76
	.DB blank, blank, blank, blank, blank, blank, blank, blank, $76
	.DB blank, blank, blank, blank, blank, blank, blank, blank, $76
	.DB blank, blank, blank, jetA1, jetB1, blank, blank, blank, $76
	.DB blank, blank, blank, jetA2, jetB2, blank, blank, blank, $76
	.DB blank, blank, blank, jetA3, jetB3, blank, blank, blank, $76
	.DB blank, blank, blank, jetA4, jetB4, blank, blank, blank, $76
	.DB blank, flrB1, blank, jetA1, jetB1, blank, blank, blank, $76
	.DB blank, flrB2, blank, jetA2, jetB2, blank, blank, blank, $76
	.DB flrA1, flrB3, flrA1, jetA3, jetB3, flrA1, flrA1, flrA1, $76
	.DB flrA2, flrB4, flrA2, jetA4, jetB4, flrA2, flrA2, flrA2, $76

	.DB splt1, splt1, splt1, splt1, splt1, splt1, splt1, splt1
        .DB $76
	.ASC " AA     BEAM <          >       "
        .DB $76
	.ASC "   1P-   1800   HI- 174500      "
        .DB $76
	.DB splt2, splt2, splt2, splt2, splt2, splt2, splt2, splt2
        .DB $76

VARAREA	.DB $80

.ENDS
